import { formatPlugin } from "./format/texter.js";
import { calcPlugin } from "./calc/calc.js";
import { utcPlugin } from "./timezone/dateutc.js";
import { tzPlugin } from "./timezone/timezone.js";
import { localePlugin } from "./locale/locale.js";
class CDateCore {
    /**
     * the constructor
     */
    constructor(t, x) {
        this.t = t;
        if ("number" !== typeof t) {
            this.d = t;
        }
        this.x = x || {};
    }
    /**
     * creates another CDate object
     */
    cdate(dt) {
        if (dt == null) {
            dt = new Date(); // now
        }
        else if ("string" === typeof dt) {
            dt = new Date(dt); // parse ISO string
        }
        else {
            dt = new Date(+dt); // number or DateLike
        }
        return this.create(dt);
    }
    /**
     * returns a read-write version of DateLike for manipulation
     */
    rw() {
        const t = +this.t;
        const rw = this.x.rw;
        return rw ? rw(t) : new Date(t);
    }
    /**
     * returns a read-only version of DateLike for displaying
     */
    ro() {
        return this.d || (this.d = this.rw());
    }
    /**
     * returns milliseconds since the epoch
     */
    valueOf() {
        return +this.ro();
    }
    /**
     * returns a raw Date object
     */
    toDate() {
        return new Date(+this);
    }
    /**
     * returns a JSON string
     */
    toJSON() {
        return this.toDate().toJSON();
    }
    plugin(fn) {
        const CDateClass = this.constructor;
        const CDateX = fn(CDateClass) || CDateClass;
        return new CDateX(this.t, this.x);
    }
    /**
     * creates another CDate object with the DateLike given
     */
    create(dt) {
        return new this.constructor(dt, this.x);
    }
    /**
     * clones the CDate object
     */
    inherit() {
        const out = this.create(+this);
        // x is readonly
        out.x = Object.create(out.x);
        return out;
    }
}
const root = new CDateCore(0, {})
    .plugin(formatPlugin)
    .plugin(calcPlugin)
    .plugin(localePlugin)
    .plugin(utcPlugin)
    .plugin(tzPlugin);
export const cdate = (dt) => root.cdate(dt);
